#include <sys/time.h>
#include <unistd.h>

#include "main.h"

#define DIN                 2               /* COMP1: 0  COMP2: 1 SVHS : 2 */
#define FILE_NAME_STYLE    "images/image%03d.ppm"

bool device_open(char *, int *, int);
void device_close(int *);
void get_image(int *);

bool initialize(void);

void file_save(int);
unsigned char bound(int, int, int);
RGB yuv2rgb(unsigned char, unsigned char, unsigned char);

struct DATA      data;
struct PIC_SIZE  pic;
struct RGB       rgb;


int main(int argc, char **argv)
{
  int video, i;
  double sec;
  bool ok;
  timeval t1,t2;

  pic.num = 10;

  if(argc>2) { pic.width = atoi(argv[1]); pic.height = atoi(argv[2]); }
  else{pic.width = WIDTH; pic.height = HEIGHT; }

  /* device init */
  ok = device_open("/dev/video", &video, DIN);
  if(!ok){ printf("Capture init failed.\n"); exit(1);}

  /* mem init */
  ok = initialize();
  if(!ok){ printf("Mem init failed.\n"); exit(1);}
  for(i=0;i<10;i++){
    get_image(&video);
  }


  // Capture the frames
  printf("Capturing\n");

  gettimeofday(&t1,NULL);
  
  for(i=0;i<pic.num;i++){
    get_image(&video);
    memcpy(data.buf + pic.width * pic.height * 2 * i,data.pdata,sizeof(unsigned char) * pic.width * pic.height * 2);
    if(i % 10 == 0){printf(".");fflush(stdout);}
  }
  
  gettimeofday(&t2,NULL);
  sec = (t2.tv_sec - t1.tv_sec) + (t2.tv_usec - t1.tv_usec) / 1E6;

  printf("\n");
  printf("Elapsed time: %4.3fs (%ffps)\n",sec,pic.num / sec);      

  printf("Writing\n");
  for(i=0;i<pic.num;i++){
    file_save(i);
  }
  printf("done.\n");

  device_close(&video);
  delete [] data.pdata;
  
  return 0;
}


bool initialize()
{
  
  /* ᥤ */
  data.pdata = new unsigned char [pic.width * pic.height * 2];
  if(!data.pdata){ printf("Not enough memory"); return (false);}
  memset(data.pdata,0,pic.width * pic.height * 2);
  
  /* Хåե */
  data.buf = new unsigned char [pic.width * pic.height * 2 * pic.num];
  if(!data.buf){ printf("Not enough memory"); return (false);}
  memset(data.buf,0,pic.width * pic.height * 2 * pic.num);

  /* RGB image */
  data.img = new unsigned char[pic.width * pic.height * 3 ];
  memset(data.img,0,pic.width * pic.height * 3);

  return (true);
}


unsigned char bound(int low,int high, int n)
{
  if(n < low ) n = low;
  if(n > high) n = high;
  return(n);
}

RGB yuv2rgb(unsigned char y, unsigned char u, unsigned char v)
{

  rgb.r = bound(0,255,(int)(1.164*(y-16.0)+1.596*(u-128.0)));
  rgb.g = bound(0,255,(int)(1.164*(y-16.0)-0.813*(u-128.0)-0.391*(v-128.0)));
  rgb.b = bound(0,255,(int)(1.164*(y-16.0)                +2.018*(v-128.0)));

  return rgb;
}


void file_save(int i)
{
  int j;
  unsigned char *p1, *p2;
  char filename[32];
  FILE *file;
    
  if(i % 10 == 0){printf(".");fflush(stdout);}

  for(j=0, p1=data.buf + pic.width * pic.height * 2 * i,p2 = data.img; j<pic.width * pic.height / 2; j++, p1+=4, p2+=6) {
    
    yuv2rgb(p1[0],p1[3],p1[1]);
    p2[0] = rgb.r; p2[1] = rgb.g; p2[2] = rgb.b;
    yuv2rgb(p1[2],p1[3],p1[1]);
    p2[3] = rgb.r; p2[4] = rgb.g; p2[5] = rgb.b;
  }  

    sprintf(filename,FILE_NAME_STYLE,i);
    if((file = fopen(filename,"w")) == NULL){ printf("file open failed.\n");}
    fprintf(file,"P6\n%d %d\n255\n",pic.width,pic.height);
    fwrite(data.img,sizeof(unsigned char),pic.width * pic.height*3,file);
    fclose(file);
  
}






